defmodule DecentralisedBookIndex.Metadata.AuthorTest do
  use DecentralisedBookIndex.DataCase, async: true

  alias DecentralisedBookIndex.Metadata

  describe "authors and registries relationship" do
    test "a new author get new registry by default" do
      assert {:ok, author} = Metadata.create_author("Author", "An description")
      assert author.author_alias_registry_id != nil
    end

    test "a new author belongs to a registry if specified" do
      assert {:ok, registry} = Metadata.create_author_alias_registry()
      assert {:ok, author} = Metadata.create_author("Author", "An description", registry.id)
      assert author.author_alias_registry_id == registry.id
    end

    test "a new author to a registry via a related author record" do
      {:ok, related_author} = Metadata.create_author("Author",  "An description")

      assert {:ok, author} =
        Metadata.add_author_to_related_alias_registry("Author2", "An description2", related_author.id)

      assert related_author.author_alias_registry_id == author.author_alias_registry_id
    end

    test "a new author to a registry via a empty related author record" do
      assert {:error, _} =
        Metadata.add_author_to_related_alias_registry("Author2", "An description2", nil)
    end
  end

  describe "authors alternatives names" do
    test "new author has no alternatives names" do
      {:ok, author} = Metadata.create_author("Author", "An description")
      assert {:ok, alternatives_names} = Metadata.get_author_alternative_names(author)
      assert alternatives_names = []
    end

    test "author has related author so they has one alternative name" do
      {:ok, related_author} = Metadata.create_author("Author",  "An description")

      {:ok, author} =
        Metadata.add_author_to_related_alias_registry("Author2", "An description2", related_author.id)

      assert {:ok, alternatives_names} = Metadata.get_author_alternative_names(author)
      assert alternatives_names = [related_author]
    end
  end

  describe "author's ids" do
    test "the list has the author's id" do
      {:ok, author} = Metadata.create_author("Author", "An description")

      assert {:ok, ids} = Metadata.get_author_ids(author)
      assert author.id in ids
    end

    test "the list has the aliases' ids" do
      {:ok, author} = Metadata.create_author("Author", "An description")
      {:ok, alias1} =
        Metadata.add_author_to_related_alias_registry("Author2", "An description2", author.id)
      {:ok, alias2} =
        Metadata.add_author_to_related_alias_registry("Author3", "An description3", author.id)

      assert {:ok, ids} = Metadata.get_author_ids(author)

      assert author.id in ids
      assert alias1.id in ids
      assert alias2.id in ids
    end
  end
end
